from .DEVICE import DEVICE

_globalVar={} # 全局变量, 为避免过多及混乱, 全部储存在一个字典里
# 为保证安全, 在外部不可直接调用 globalVar, 必须通过函数调用
# 在外部调用 get 开头的函数时, 必须确保结果是只读的, 否则会对这里的可变对象的值产生影响

_globalVar['device']=DEVICE # 本地电脑为1, 服务器为0, 该值不可在函数中修改



def getGlobalValue(key):
    global _globalVar
    return _globalVar[key]


def setGlobalValue(key,v):
    global _globalVar
    _globalVar[key]=v


def getAllGlobalVariables():
    global _globalVar
    return _globalVar

def getGlobalVariableNum():
    global _globalVar
    return len(_globalVar)

def clearGlobalVariable():
    global _globalVar
    _globalVar={}

def deleteGlobalVariable(key):
    global _globalVar
    _globalVar.pop(key)


def getAllKeys():
    global _globalVar
    l=[]
    for k in _globalVar:
        l.append(k)
    return l

def isGlobalVariableExist(key):
    global _globalVar
    return key in _globalVar


def addGlobalVariable(key,num=1):
    global _globalVar
    try:
        _globalVar[key]=_globalVar[key]+num
    except:
        return -1
    return 0



from .myHttp import *
from .others import *
import time
import json
import base64
import hmac

def _okexBasic_getFormatTime():
    t=time.gmtime()[0:6]
    ft=str(t[0])+'-'
    if(t[1]<=9):
        ft=ft+'0'
    ft=ft+str(t[1])+'-'
    if(t[2]<=9):
        ft=ft+'0'
    ft=ft+str(t[2])+'T'
    if(t[3]<=9):
        ft=ft+'0'
    ft=ft+str(t[3])+':'
    if(t[4]<=9):
        ft=ft+'0'
    ft=ft+str(t[4])+':'
    if(t[5]<=9):
        ft=ft+'0'
    ft=ft+str(t[5])
    ft=ft+'.'+str(getTime()%1000)+'Z'
    return ft

def _okexBasic_sha256(Text:str,Key:str): # 已验证没有问题
    sign=hmac.new(Key.encode('utf-8'),Text.encode('utf-8'),digestmod='sha256').digest()
    sign=base64.b64encode(sign)
    sign=str(sign,'utf-8')
    return sign




def okGet(BasicUrl:str,Parameters:dict,TimeOut:int=0): # 这是不需要签名的
    timeOut=TimeOut
    while(BasicUrl.find('<')>=0):
        n1=BasicUrl.find('<')
        n2=BasicUrl.find('>')
        name=BasicUrl[n1+1:n2]
        value=str(Parameters[name])
        Parameters.pop(name)
        BasicUrl=BasicUrl[0:n1]+value+BasicUrl[n2+1:]
    url=BasicUrl
    if(len(Parameters)!=0):
        url=url+'?'
        for k,v in Parameters.items():
            url=url+str(k)+'='+str(v)+'&'
        url=url[:-1]
    r=http(url,Timeout=timeOut)
    return r

def okSign(BasicUrl:str,APIList:list,Parameters:dict,TimeOut:int=0,isSimlated:bool=False):
    # GET  APIList=[APIKey,SecretKey,PassPhrase]
    header={}
    t=_okexBasic_getFormatTime()
    header['Content-Type']='application/json'
    header['OK-ACCESS-KEY']=APIList[0]
    header['OK-ACCESS-PASSPHRASE']=APIList[2]
    header['OK-ACCESS-TIMESTAMP']=t
    if(isSimlated):
        header['x-simulated-trading']=1
    while(BasicUrl.find('<')>=0):
        n1=BasicUrl.find('<')
        n2=BasicUrl.find('>')
        name=BasicUrl[n1+1:n2]
        value=str(Parameters[name])
        Parameters.pop(name)
        BasicUrl=BasicUrl[0:n1]+value+BasicUrl[n2+1:]
    url=BasicUrl
    if(len(Parameters)!=0):
        url=url+'?'
        for k,v in Parameters.items():
            url=url+str(k)+'='+str(v)+'&'
        url=url[:-1]
    n=url.find('/',5+url.find('://'))
    sigUrl=url[n:]
    sigStr=t+'GET'+sigUrl
    sig=_okexBasic_sha256(sigStr,APIList[1])
    header['OK-ACCESS-SIGN']=sig
    r=http(url,Header=header,Timeout=TimeOut)
    return r


def okPost(BasicUrl:str,APIList:list,Parameters:dict,TimeOut:int=0,isSimlated:bool=False):
    header={}
    t=_okexBasic_getFormatTime()
    header['Content-Type']='application/json'
    header['OK-ACCESS-KEY']=APIList[0]
    header['OK-ACCESS-PASSPHRASE']=APIList[2]
    header['OK-ACCESS-TIMESTAMP']=t
    if(isSimlated):
        header['x-simulated-trading']=1
    while(BasicUrl.find('<')>=0):
        n1=BasicUrl.find('<')
        n2=BasicUrl.find('>')
        name=BasicUrl[n1+1:n2]
        value=str(Parameters[name])
        Parameters.pop(name)
        BasicUrl=BasicUrl[0:n1]+value+BasicUrl[n2+1:]
    body=json.dumps(Parameters)
    url=BasicUrl
    n=url.find('/',5+url.find('://'))
    sigUrl=url[n:]
    sigStr=t+'POST'+sigUrl+body
    sig=_okexBasic_sha256(sigStr,APIList[1])
    header['OK-ACCESS-SIGN']=sig
    r=http(url,Method='POST',Header=header,Timeout=TimeOut,BODY=body)
    return r





from .myHttp import *
from .others import *
import time 
import hmac
import base64
import json
from urllib.parse import quote


def _huobiBasic_getFormatTime():
    t=time.gmtime()[0:6]
    ft=str(t[0])+'-'
    if(t[1]<=9):
        ft=ft+'0'
    ft=ft+str(t[1])+'-'
    if(t[2]<=9):
        ft=ft+'0'
    ft=ft+str(t[2])+'T'
    if(t[3]<=9):
        ft=ft+'0'
    ft=ft+str(t[3])+'%3A'
    if(t[4]<=9):
        ft=ft+'0'
    ft=ft+str(t[4])+'%3A'
    if(t[5]<=9):
        ft=ft+'0'
    ft=ft+str(t[5])
    return ft

def _huobiBasic_sha256(Text:str,Key:str): # 已验证没有问题
    sign=hmac.new(Key.encode('utf-8'),Text.encode('utf-8'),digestmod='sha256').digest()
    sign=base64.b64encode(sign)
    sign=str(sign,'utf-8')
    sign=quote(sign,'utf-8')
    return sign


def hbGet(BasicUrl:str,Parameters:dict,TimeOut:int=0): # 这是不需要签名的
    # 不需要签名，参数无顺序要求
    basicUrl=BasicUrl
    parameters=Parameters
    timeOut=TimeOut
    url=basicUrl
    if(len(parameters)!=0):
        url=url+'?'
        for k,v in parameters.items():
            url=url+str(k)+'='+str(v)+'&'
        url=url[:-1]
    r=http(url,Timeout=timeOut)
    return r


#def hbSign(BasicUrl:str,API_Key:str,SecretKey:str,Parameters:dict,TimeOut:int=0):
def hbSign(BasicUrl:str,APIList:list,Parameters:dict,TimeOut:int=0):
    # GET
    API_Key=APIList[0]
    SecretKey=APIList[1]
    if(BasicUrl.find('{')>=0):
        n1=BasicUrl.find('{')
        n2=BasicUrl.find('}')
        name=BasicUrl[n1+1:n2]
        value=str(Parameters[name])
        Parameters.pop(name)
        BasicUrl=BasicUrl[0:n1]+value+BasicUrl[n2+1:]
    Parameters['AccessKeyId']=API_Key
    Parameters['SignatureMethod']='HmacSHA256'
    Parameters['SignatureVersion']=2
    Parameters['Timestamp']=_huobiBasic_getFormatTime()
    paraList=[]
    for k,v in Parameters.items():
        paraList.append(str(k)+'='+str(v))
    paraList.sort()
    paraStr=''
    for i in range(0,len(paraList)):
        paraStr=paraStr+paraList[i]+'&'
    paraStr=paraStr[0:-1]
    n=BasicUrl.find('://')
    url=BasicUrl[n+3:]
    n=url.find('/')
    domain=url[0:n]
    aft=url[n:]
    sigStr="GET\n"+domain+'\n'+aft+'\n'+paraStr
    sig=_huobiBasic_sha256(sigStr,SecretKey)
    paraStr=paraStr+'&Signature='+sig
    url=BasicUrl+'?'+paraStr
    r=http(url,Timeout=TimeOut)
    return r

#def hbPost(BasicUrl:str,API_Key:str,SecretKey:str,Parameters:dict,TimeOut:int=0):
def hbPost(BasicUrl:str,APIList:list,Parameters:dict,TimeOut:int=0):
    API_Key=APIList[0]
    SecretKey=APIList[1]
    if(BasicUrl.find('{')>=0):
        n1=BasicUrl.find('{')
        n2=BasicUrl.find('}')
        name=BasicUrl[n1+1:n2]
        value=str(Parameters[name])
        Parameters.pop(name)
        BasicUrl=BasicUrl[0:n1]+value+BasicUrl[n2+1:]
    urlPara={}
    urlPara['AccessKeyId']=API_Key
    urlPara['SignatureMethod']='HmacSHA256'
    urlPara['SignatureVersion']=2
    urlPara['Timestamp']=_huobiBasic_getFormatTime()
    urlParaList=[]
    for k,v in urlPara.items():
        urlParaList.append(str(k)+'='+str(v))
    urlParaList.sort()
    urlParaStr=''
    for i in range(0,len(urlParaList)):
        urlParaStr=urlParaStr+urlParaList[i]+'&'
    urlParaStr=urlParaStr[0:-1]
    n=BasicUrl.find('://')
    url=BasicUrl[n+3:]
    n=url.find('/')
    domain=url[0:n]
    aft=url[n:]
    sigStr="POST\n"+domain+'\n'+aft+'\n'+urlParaStr
    sig=_huobiBasic_sha256(sigStr,SecretKey)
    urlParaStr=urlParaStr+'&Signature='+sig
    url=BasicUrl+'?'+urlParaStr
    paraStr=json.dumps(Parameters)
    header={'Content-Type':'application/json'}
    r=http(url,Method='POST',Timeout=TimeOut,BODY=paraStr,Header=header)
    return r




from .binanceBasic import bnGet,bnPPD,bnSign
from .huobiBasic import hbGet,hbPost,hbSign
from .okexBasic import okGet,okPost,okSign
from .krakenBasic import kkGet,kkPost,kkSign

__all__=[
    'bnGet','bnPPD','bnSign',
    'hbGet','hbPost','hbSign',
    'okGet','okPost','okSign',
    'kkGet','kkPost','kkSign'
]


__version__='0.0.3'
__license__='GPL-2.0 License'
__author__='Tiancheng Jiao'
__url__='https://github.com/jtc1246/cryptoEx'
__author_email__='jtc1246@outlook.com'
__description__='A simple API for some cryptocurrency exchanges'


import time
import json
import hmac

def getTime():
    t =time.time()
    t = int(1000 * t)
    return t


def toJson(Text):
    text=Text
    i=0
    while text[0]==' ' or text[0]=='\n':
        text=text[1:]
    if text[0]=='[':
        text='{"1":'+text+'}'
        i=1
    text=json.loads(text,strict=False)
    if i==1:
        text=text['1']
    return text

def sha256(Text:str,Key:str):
    sign=hmac.new(Key.encode('utf-8'),Text.encode('utf-8'),digestmod='sha256').hexdigest()
    return sign




from .myHttp import *
from .others import *
import urllib.parse as parse
import hashlib
import hmac
import base64




def _krakenBasic_get_kraken_signature(urlpath, data, secret):
    # 直接从 API 文档复制的
    postdata = parse.urlencode(data)
    encoded = (str(data['nonce']) + postdata).encode()
    message = urlpath.encode() + hashlib.sha256(encoded).digest()
    #print(message)
    mac = hmac.new(base64.b64decode(secret), message, hashlib.sha512)
    sigdigest = base64.b64encode(mac.digest())
    return sigdigest.decode()

def _krakenBasic_isFuture(url:str):
    return url.find('futures.')>=0


def _krakenBasic_sign_message(endpoint, postData, SecretKey,nonce):
    if endpoint.startswith('/derivatives'):
        endpoint = endpoint[len('/derivatives'):]
    nonce=str(nonce)
    # step 1: concatenate postData, nonce + endpoint
    message = postData + nonce + endpoint
    # step 2: hash the result of step 1 with SHA256
    sha256_hash = hashlib.sha256()
    sha256_hash.update(message.encode('utf8'))
    hash_digest = sha256_hash.digest()
    # step 3: base64 decode apiPrivateKey
    secretDecoded = base64.b64decode(SecretKey)
    # step 4: use result of step 3 to has the result of step 2 with HMAC-SHA512
    hmac_digest = hmac.new(secretDecoded, hash_digest,
                           hashlib.sha512).digest()
    # step 5: base64 encode the result of step 4 and return
    return base64.b64encode(hmac_digest)



def kkGet(BasicUrl:str,Parameters:dict,TimeOut:int=0): # 不需要签名的
    while(BasicUrl.find('{')>=0):
        n1=BasicUrl.find('{')
        n2=BasicUrl.find('}')
        name=BasicUrl[n1+1:n2]
        value=str(Parameters[name])
        Parameters.pop(name)
        BasicUrl=BasicUrl[0:n1]+value+BasicUrl[n2+1:]
    basicUrl=BasicUrl
    parameters=Parameters
    timeOut=TimeOut
    url=basicUrl
    if(len(parameters)!=0):
        url=url+'?'
        for k,v in parameters.items():
            url=url+str(k)+'='+str(v)+'&'
        url=url[:-1]
    r=http(url,Timeout=timeOut)
    return r


def kkPost(BasicUrl:str,APIList:list,Parameters:dict,TimeOut:int=0):
    if(_krakenBasic_isFuture(BasicUrl)):
        return _krakenBasic_futurePost(BasicUrl,APIList=APIList,Parameters=Parameters,TimeOut=TimeOut)
    header={}
    header['API-Key']=APIList[0]
    header['Content-Type']='application/x-www-form-urlencoded; charset=utf-8'
    Parameters['nonce']=getTime()
    n=BasicUrl.find('://')
    n=BasicUrl.find('/',n+5)
    urlpath=BasicUrl[n:]
    sig=_krakenBasic_get_kraken_signature(urlpath,Parameters,APIList[1])
    header['API-Sign']=sig
    body=''
    if(len(Parameters)!=0):
        for k,v in Parameters.items():
            body=body+str(k)+'='+str(v)+'&'
        body=body[0:-1]
    r=http(BasicUrl,Method='POST',Header=header,Timeout=TimeOut,BODY=body)
    return r


def kkSign(BasicUrl:str,APIList:list,Parameters:dict,TimeOut:int=0):
    # GET 且需要签名的
    header={}
    header["User-Agent"] = "cf-api-python/1.0"
    non=str(getTime())
    header['APIKey']=APIList[0]
    header['Nonce']=non
    n=BasicUrl.find('://')
    n=BasicUrl.find('/',n+5)
    urlpath=BasicUrl[n:]
    paraStr=''
    if(len(Parameters)!=0):
        for k,v in Parameters.items():
            paraStr=paraStr+str(k)+'='+str(v)+'&'
        paraStr=paraStr[0:-1]
    sig=_krakenBasic_sign_message(urlpath,paraStr,APIList[1],non)
    header['Authent']=sig
    url=BasicUrl
    if paraStr!='':
        url=BasicUrl+'?'+paraStr
    r=http(url,Header=header,Timeout=TimeOut)
    return r


def _krakenBasic_futurePost(BasicUrl:str,APIList:list,Parameters:dict,TimeOut:int=0):
    header={}
    header["User-Agent"] = "cf-api-python/1.0"
    non=str(getTime())
    header['APIKey']=APIList[0]
    header['Nonce']=non
    n=BasicUrl.find('://')
    n=BasicUrl.find('/',n+5)
    urlpath=BasicUrl[n:]
    paraStr=''
    if(len(Parameters)!=0):
        for k,v in Parameters.items():
            paraStr=paraStr+str(k)+'='+str(v)+'&'
        paraStr=paraStr[0:-1]
    sig=_krakenBasic_sign_message(urlpath,paraStr,APIList[1],non)
    header['Authent']=sig
    url=BasicUrl
    if paraStr!='':
        url=BasicUrl+'?'+paraStr
    r=http(url,Method='POST',Header=header,Timeout=TimeOut)
    return r




from setuptools import setup

setup(
    name='cryptoEx',
    version='0.0.3',
    author='Tiancheng Jiao',
    author_email='jtc1246@outlook.com',
    url='https://github.com/jtc1246/cryptoEx',
    description='A simple API for some cryptocurrency exchanges',
    packages=['cryptoEx'],
    install_requires=['urllib3'],
    python_requires='>=3',
    platforms=["all"],
    license='GPL-2.0 License'
)


from .myHttp import *
from .others import *

def bnGet(BasicUrl:str,Parameters:dict,API_Key:str='',TimeOut:int=0): # 这是不需要签名的
    basicUrl=BasicUrl
    parameters=Parameters
    APIKey=API_Key
    timeOut=TimeOut
    url=basicUrl
    if(len(parameters)!=0):
        url=url+'?'
        for k,v in parameters.items():
            url=url+str(k)+'='+str(v)+'&'
        url=url[:-1]
    header={}
    if(APIKey!=''):
        header={'X-MBX-APIKEY':APIKey}
    r=http(url,Header=header,Timeout=timeOut)
    return r

def bnSign(BasicUrl:str,API_Key:str,SecretKey:str,Parameters:dict,TimeOut:int=0):
    # GET 且需要签名的
    header={'X-MBX-APIKEY':API_Key}
    body=''
    for k,v in Parameters.items():
        body=body+str(k)+'='+str(v)+'&'
    body=body+'timestamp='+str(getTime())
    sig=sha256(body,SecretKey)
    body=body+'&signature='+sig
    url=BasicUrl+'?'+body
    r=http(url,Header=header,Timeout=TimeOut)
    return r

def bnPPD(BasicUrl:str,API_Key:str,SecretKey:str,Parameters:dict,Method:str='POST',TimeOut:int=0):
    # PPD: POST, PUT, DELETE
    m=Method
    body=''
    for k,v in Parameters.items():
        body=body+str(k)+'='+str(v)+'&'
    body=body+'timestamp='+str(getTime())
    sig=sha256(body,SecretKey)
    body=body+'&signature='+sig
    header={'X-MBX-APIKEY':API_Key,'Content-Type':'application/x-www-form-urlencoded'}
    r=http(BasicUrl,Header=header,Timeout=TimeOut,BODY=body,Method=m)
    return r

